@@ -1,4 +1,4 @@
-# (C) Copyright 2016 Hewlett Packard Enterprise Development Company LP
+# (C) Copyright 2016-2017 Hewlett Packard Enterprise Development LP
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,13 +12,12 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from datetime import datetime
 import json
 
 from oslo_log import log
-import pytz
 
 from monasca_persister.repositories.influxdb import abstract_repository
+from monasca_persister.repositories.influxdb import line_utils
 from monasca_persister.repositories.utils import parse_measurement_message
 
 LOG = log.getLogger(__name__)
@@ -36,20 +35,27 @@ class MetricInfluxdbRepository(abstract_repository.AbstractInfluxdbRepository):
          value_meta) = parse_measurement_message(message)
 
         tags = dimensions
-        tags['_tenant_id'] = tenant_id.encode('utf8')
-        tags['_region'] = region.encode('utf8')
+        tags[u'_tenant_id'] = tenant_id
+        tags[u'_region'] = region
 
-        ts = time_stamp / 1000.0
+        if not value_meta:
+            value_meta_str = u'"{}"'
+        else:
+            value_meta_str = line_utils.escape_value(json.dumps(value_meta, ensure_ascii=False))
 
-        data = {"measurement": metric_name.encode('utf8'),
-                "time": datetime.fromtimestamp(ts, tz=pytz.utc).strftime(
-                        '%Y-%m-%dT%H:%M:%S.%fZ'),
-                "fields": {
-                    "value": value,
-                    "value_meta": json.dumps(value_meta,
-                                             ensure_ascii=False).encode('utf8')
-                },
-                "tags": tags}
+        key_values = [line_utils.escape_tag(metric_name)]
+
+        # tags should be sorted client-side to take load off server
+        for key in sorted(tags.keys()):
+            key_tag = line_utils.escape_tag(key)
+            value_tag = line_utils.escape_tag(tags[key])
+            key_values.append(key_tag + u'=' + value_tag)
+        key_values = u','.join(key_values)
+
+        value_field = u'value={}'.format(value)
+        value_meta_field = u'value_meta=' + value_meta_str
+
+        data = key_values + u' ' + value_field + u',' + value_meta_field + u' ' + str(int(time_stamp))
 
         LOG.debug(data)
 
